<?php

namespace Drupal\cforge\Plugin\Mail;

use Drupal\Core\Mail\Plugin\Mail\PhpMail;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Mail\MailFormatHelper;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Defines a mail backend that prints mails to screen in demo_mode
 *
 * @Mail(
 *   id = "cforge_mailer",
 *   label = @Translation("Hamlets Mailer"),
 *   description = @Translation("Sends rich text email")
 * )
 */
class CforgeMailer extends PhpMail implements ContainerFactoryPluginInterface {

  protected $configFactory;
  protected $messenger;

  /**
   * @param ConfigFactoryInterface $config_factory
   * @param MessengerInterface $messenger
   */
  public function __construct($configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
  }

    /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('config.factory'),
      $container->get('messenger')
    );
  }



  public function format(array $message) {
    $eol = PHP_EOL;
    $boundary = uniqid('np');
    $message['headers']['Sender'] = '"'
      . str_replace('"', '', $this->configFactory()->get('system.site')->get('mail'))
      . '" <' . $default_from . '>';


    // Join the body array into one string.
    $html = implode("\n\n", $message['body']);

    $message['headers']['Content-Type'] = 'multipart/alternative;boundary="' . $boundary . '"';
    //$message['headers']['MIME-Version'] = '1.0';

    $raw_message = 'This is a MIME encoded message.';
    $raw_message .= $eol . $eol . "--" . $boundary . $eol;
    $raw_message .= "Content-Type: text/plain;charset=utf-8" . $eol . $eol;
    $raw_message .= MailFormatHelper::htmlToText($html);
    $raw_message .= $eol . $eol . "--" . $boundary . $eol;
    $raw_message .= "Content-Type: text/html;charset=utf-8" . $eol . $eol;
    $raw_message .= $html;
    $raw_message .= $eol . $eol . "--" . $boundary . "--";
    $message['body'] = $raw_message;

    return $message;
  }

  /**
   * {@inheritdoc}
   *
   * Don't send mail to example users.
   */
  public function mail(array $message) {
    if (1 or stripos($message['to'], 'example.com')) {
      $output[] = "to: ".$message['to'];
      $output[] = "from: ".$message['from'];
      foreach ($message['headers'] as $key => $value) {
        $output[] = "$key: $value";
      }
      $output[] = '';
      $output[] = $message['body'];
      // This works for plaintext mails.
      $markup = \Drupal\Core\Render\Markup::create('<pre>'.implode('<br>', $output).'</pre>');
      $this->messenger->addStatus($markup);
      return TRUE;
    }
    return parent::mail($message);
  }

}
